@@ -21,6 +21,7 @@ pub mod fmt;
 pub mod frequency;
 pub mod from;
 pub mod fuzzy_join;
+pub mod grep;
 pub mod groupby;
 pub mod head;
 pub mod headers;
@@ -46,6 +47,7 @@ pub mod sample;
 pub mod scrape;
 pub mod search;
 pub mod select;
+pub mod separate;
 pub mod shuffle;
 pub mod slice;
 pub mod sort;
