@@ -2,13 +2,12 @@ use std::io::{stdout, Write};
 
 use crate::config::{Config, Delimiter};
 use crate::moonblade::PivotAggregationProgram;
-use crate::select::{SelectColumns, Selection};
+use crate::select::{SelectedColumns, Selection};
 use crate::util;
 use crate::CliResult;
 
 // TODO: optimize when names are known beforehand
 // TODO: -S/--sorted
-// TODO: multiselections
 
 static USAGE: &str = r#"
 Pivot a CSV file by allowing distinct values from a column to be separated into
@@ -58,14 +57,19 @@ you can easily call `xan pivot` downstream of `xan unpivot`:
     $ xan unpivot january: monthly.csv | <processing> | xan pivot
 
 Usage:
-    xan pivot [-P...] [options] <column> <expr> [<input>]
+    xan pivot [-P...] [options] <columns> <expr> [<input>]
+    xan pivot-wider [-P...] [options] <columns> <expr> [<input>]
     xan pivot [-P...] [options] [<input>]
+    xan pivot-wider [-P...] [options] [<input>]
     xan pivot --help
+    xan pivot-wider --help
 
 pivot options:
     -g, --groupby <columns>  Group results by given selection of columns instead
                              of grouping by columns not used to pivot nor in
                              aggregation.
+    --column-sep <sep>       Separator used to join column names when pivoting
+                             on multiple columns. [default: _]
 
 pivotal options:
     -P  Use at least three times to get help from your friends!
@@ -82,9 +86,10 @@ Common options:
 #[derive(Deserialize, Debug)]
 struct Args {
     arg_input: Option<String>,
-    arg_column: Option<SelectColumns>,
+    arg_columns: Option<SelectedColumns>,
     arg_expr: Option<String>,
-    flag_groupby: Option<SelectColumns>,
+    flag_groupby: Option<SelectedColumns>,
+    flag_column_sep: String,
     #[serde(rename = "flag_P")]
     flag_p: usize,
     flag_output: Option<String>,
@@ -106,8 +111,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     // Handling defaults
     let arg_column = args
-        .arg_column
-        .unwrap_or_else(|| SelectColumns::parse("name").unwrap());
+        .arg_columns
+        .unwrap_or_else(|| SelectedColumns::parse("name").unwrap());
 
     let arg_expr = args.arg_expr.unwrap_or_else(|| "first(value)".to_string());
 
@@ -116,22 +121,25 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .delimiter(args.flag_delimiter)
         .select(arg_column);
 
-    let mut rdr = rconf.reader()?;
+    let mut rdr = rconf.simd_reader()?;
     let headers = rdr.byte_headers()?.clone();
-    let pivot_col_index = rconf.single_selection(&headers)?;
+    let pivot_sel = rconf.selection(&headers)?;
     let mut program = PivotAggregationProgram::parse(&arg_expr, &headers)?;
 
     let column_indices_used_in_aggregation = program.used_column_indices();
 
-    if column_indices_used_in_aggregation.contains(&pivot_col_index) {
+    if column_indices_used_in_aggregation
+        .iter()
+        .any(|i| pivot_sel.contains(*i))
+    {
         Err("aggregation cannot work on the pivot column!")?;
     }
 
-    let mut disappearing_columns = vec![pivot_col_index];
+    let mut disappearing_columns = pivot_sel.to_vec();
     disappearing_columns.extend(column_indices_used_in_aggregation);
 
     let groupby_sel = if let Some(cols) = args.flag_groupby.as_ref() {
-        let sel = cols.selection(&headers, !args.flag_no_headers)?;
+        let sel = cols.selection(&headers, !rconf.no_headers)?;
 
         if sel.iter().any(|i| disappearing_columns.contains(i)) {
             Err(
@@ -144,14 +152,14 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         Selection::without_indices(headers.len(), &disappearing_columns)
     };
 
-    let mut wtr = Config::new(&args.flag_output).writer()?;
+    let mut wtr = Config::new(&args.flag_output).simd_writer()?;
 
     let mut index: usize = 0;
-    let mut record = csv::ByteRecord::new();
+    let mut record = simd_csv::ByteRecord::new();
 
     while rdr.read_byte_record(&mut record)? {
         let group = groupby_sel.collect(&record);
-        let pivot = record[pivot_col_index].to_vec();
+        let pivot = bstr::join(&args.flag_column_sep, pivot_sel.collect(&record));
 
         program.run_with_record(group, pivot, index, &record)?;
 
@@ -159,7 +167,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     }
 
     if !rconf.no_headers {
-        let mut output_headers = groupby_sel.select(&headers).collect::<csv::ByteRecord>();
+        let mut output_headers = groupby_sel
+            .select(&headers)
+            .collect::<simd_csv::ByteRecord>();
 
         for name in program.pivoted_column_names() {
             output_headers.push_field(name);
