@@ -20,13 +20,13 @@ use ratatui::style::{Style, Stylize};
 use ratatui::symbols;
 use ratatui::widgets::{Axis, Chart, Dataset, GraphType};
 
-use crate::collections::HashMap;
 use crate::config::{Config, Delimiter};
 use crate::dates::{infer_temporal_granularity, parse_partial_date, parse_zoned};
+use crate::moonblade::GroupAggregationProgram;
 use crate::ratatui::print_ratatui_frame_to_stdout;
 use crate::scales::{Scale, ScaleType};
-use crate::select::SelectColumns;
-use crate::util;
+use crate::select::SelectedColumns;
+use crate::util::{self, ColorMode};
 use crate::{CliError, CliResult};
 
 const TYPICAL_COLS: usize = 35;
@@ -85,6 +85,38 @@ impl TryFrom<String> for Granularity {
 static USAGE: &str = "
 Draw a scatter plot or a line plot based on 2-dimensional data.
 
+It is also possible to draw multiple series/lines, as well as drawing multiple
+series/lines as small multiples (sometimes also called a facet grid), by providing
+a -c/--category column or selecting multiple columns as <y> series.
+
+Drawing a simple scatter plot:
+
+    $ xan plot sepal_width sepal_length iris.csv
+
+Drawing a categorical scatter plot:
+
+    $ xan plot sepal_width sepal_length -c species iris.csv
+
+The same, as small multiples:
+
+    $ xan plot sepal_width sepal_length -c species iris.csv -S 2
+
+As a line chart:
+
+    $ xan plot -L sepal_length petal_length iris.csv
+
+Plotting time series:
+
+    $ xan plot -LT datetime units sales.csv
+
+Plotting multiple comparable times series at once:
+
+    $ xan plot -LT datetime amount,amount_fixed sales.csv
+
+Different times series, as small multiples:
+
+    $ xan plot -LT datetime revenue,units sales.csv -S 2
+
 Usage:
     xan plot --count [options] <x> [<input>]
     xan plot [options] <x> <y> [<input>]
@@ -100,11 +132,15 @@ plot options:
                                y values will be summed wrt the newly discretized x axis.
     --count                    Omit the y column and count rows instead. Only relevant when
                                used with -T, --time that will discretize the x axis.
+    -A, --aggregate <expr>     Expression that will be used to aggregate values falling into
+                               the same bucket when discretizing the x axis, e.g. when using
+                               the -T, --time flag. The `_` implicit variable will be use to
+                               denote a value in said expression. For instance, if you want
+                               to average the values you can pass `mean(_)`. Will default
+                               to `sum(_)`.
     -c, --category <col>       Name of the categorical column that will be used to
                                draw distinct series per category.
-                               Incompatible with -Y, --add-series.
-    -Y, --add-series <col>     Name of another column of y values to add as a new series.
-                               Incompatible with -c, --category.
+                               Does not work when selecting multiple columns with <y>.
     -R, --regression-line      Draw a regression line. Only works when drawing a scatter plot with
                                a single series.
     -g, --granularity <g>      Force temporal granularity for x axis discretization when
@@ -119,16 +155,16 @@ plot options:
                                terminal size cannot be found (i.e. when piping to file).
                                Can also be given as a ratio of the terminal's height e.g. \"0.5\".
     -S, --small-multiples <n>  Display small multiples (also called facet grids) of datasets
-                               given by -c, --category or -Y, --add-series using the provided number
-                               of grid columns. The plot will all share the same x scale but use a different
-                               y scale by default. See --share-y-scale and --separate-x-scale to tweak
-                               this behavior.
+                               given by -c, --category or when multiple series are provided to <y>,
+                               using the provided number of grid columns. The plot will all share the same
+                               x scale but use a different y scale by default. See --share-y-scale
+                               and --separate-x-scale to tweak this behavior.
     --share-x-scale <yes|no>   Give \"yes\" to share x scale for all plot when drawing small multiples with -S,
                                or \"no\" to keep them separate.
                                [default: yes]
     --share-y-scale <yes|no>   Give \"yes\" to share y scale for all plot when drawing small multiples with -S,
                                or \"no\" to keep them separate. Defaults to \"yes\" when -c, --category is given
-                               and \"no\" when -Y, --add-series is given.
+                               and \"no\" when multiple series are provided to <y>.
     -M, --marker <name>        Marker to use. Can be one of (by order of size): 'braille', 'dot',
                                'halfblock', 'bar', 'block'.
                                [default: braille]
@@ -141,12 +177,17 @@ plot options:
     --x-max <n>                Force a maximum value for the x axis.
     --y-min <n>                Force a minimum value for the y axis.
     --y-max <n>                Force a maximum value for the y axis.
-    --x-scale <scale>          Apply a scale to the x axis. Can be one of \"lin\" or \"log\".
+    --x-scale <scale>          Apply a scale to the x axis. Can be one of \"lin\", \"log\",
+                               \"log2\", \"log10\" or \"log(custom_base)\" like \"log(2.5)\".
                                [default: lin]
-    --y-scale <scale>          Apply a scale to the y axis. Can be one of \"lin\" or \"log\".
+    --y-scale <scale>          Apply a scale to the y axis. Can be one of \"lin\", \"log\",
+                               \"log2\", \"log10\" or \"log(custom_base)\" like \"log(2.5)\".
                                [default: lin]
-    -C, --force-colors         Force colors even if output is not supposed to be able to
-                               handle them.
+    --color <when>             When to color the output using ANSI escape codes.
+                               Use `auto` for automatic detection, `never` to
+                               disable colors completely and `always` to force
+                               colors, even when the output could not handle them.
+                               [default: auto]
     -i, --ignore               Ignore values that cannot be correctly parsed.
 
 Common options:
@@ -160,21 +201,21 @@ Common options:
 #[derive(Deserialize)]
 struct Args {
     arg_input: Option<String>,
-    arg_x: SelectColumns,
-    arg_y: Option<SelectColumns>,
+    arg_x: SelectedColumns,
+    arg_y: Option<SelectedColumns>,
     flag_no_headers: bool,
     flag_delimiter: Option<Delimiter>,
     flag_line: bool,
     flag_bars: bool,
     flag_time: bool,
     flag_count: bool,
+    flag_aggregate: Option<String>,
     flag_cols: Option<String>,
     flag_rows: Option<String>,
     flag_small_multiples: Option<NonZeroUsize>,
     flag_share_x_scale: String,
     flag_share_y_scale: Option<String>,
-    flag_category: Option<SelectColumns>,
-    flag_add_series: Vec<SelectColumns>,
+    flag_category: Option<SelectedColumns>,
     flag_regression_line: bool,
     flag_marker: Marker,
     flag_granularity: Option<Granularity>,
@@ -187,7 +228,7 @@ struct Args {
     flag_y_max: Option<f64>,
     flag_x_scale: ScaleType,
     flag_y_scale: ScaleType,
-    flag_force_colors: bool,
+    flag_color: ColorMode,
     flag_ignore: bool,
 }
 
@@ -245,6 +286,7 @@ impl Args {
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
+    args.flag_color.apply();
     let rconf = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers);
@@ -253,23 +295,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         Err("--x-scale cannot be customized when using -T,--time")?;
     }
 
-    if args.flag_regression_line {
-        if args.flag_bars || args.flag_line {
-            Err("-R/--regression-line does not work with -B/--bars nor -L/--line!")?;
-        }
-
-        if args.flag_category.is_some() || !args.flag_add_series.is_empty() {
-            Err("-R/--regression-line only works with single series (avoid -c/--category and -Y/--add-series)!")?;
-        }
-    }
-
-    let share_x_scale = args.flag_share_x_scale == "yes";
-    let share_y_scale = args
-        .flag_share_y_scale
-        .as_ref()
-        .map(|choice| choice == "yes")
-        .unwrap_or(args.flag_add_series.is_empty());
-
     debug_assert!(if args.flag_count {
         args.arg_y.is_none()
     } else {
@@ -291,10 +316,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         Err("--y-min or --y-max cannot be <= 0 with --y-scale log!")?;
     }
 
-    if args.flag_category.is_some() && !args.flag_add_series.is_empty() {
-        Err("-c, --category cannot work with -Y, --add-series!")?;
-    }
-
     if matches!(args.flag_x_ticks, Some(n) if n.get() < 2) {
         Err("--x-ticks must be > 1!")?;
     }
@@ -303,25 +324,18 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         Err("--y-ticks must be > 1!")?;
     }
 
-    if args.flag_force_colors {
-        colored::control::set_override(true);
-    }
-
-    let has_added_series = !args.flag_add_series.is_empty();
-
     // Collecting data
-    let mut rdr = rconf.reader()?;
+    let mut rdr = rconf.simd_reader()?;
     let headers = rdr.byte_headers()?;
 
-    let x_column_index = args
-        .arg_x
-        .single_selection(headers, !args.flag_no_headers)?;
+    let x_column_index = args.arg_x.single_selection(headers, !rconf.no_headers)?;
 
     let y_column_index_opt = args
         .arg_y
         .as_ref()
-        .map(|name| name.single_selection(headers, !args.flag_no_headers))
-        .transpose()?;
+        .map(|name| name.selection(headers, !rconf.no_headers))
+        .transpose()?
+        .map(|s| s.into_first().unwrap());
 
     let x_column_name = if args.flag_no_headers {
         x_column_index.to_string()
@@ -347,29 +361,58 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let category_column_index = args
         .flag_category
         .as_ref()
-        .map(|name| name.single_selection(headers, !args.flag_no_headers))
+        .map(|name| name.single_selection(headers, !rconf.no_headers))
         .transpose()?;
 
     let additional_series_indices = args
-        .flag_add_series
-        .iter()
-        .map(|name| {
-            let i = name.single_selection(headers, !args.flag_no_headers)?;
+        .arg_y
+        .as_ref()
+        .map(|names| -> CliResult<Vec<(Vec<u8>, usize)>> {
+            let sel = names.selection(headers, !rconf.no_headers)?.into_rest();
 
-            let col_name = if args.flag_no_headers {
-                i.to_string().into_bytes()
+            let info: Vec<(Vec<u8>, usize)> = if args.flag_no_headers {
+                sel.iter()
+                    .map(|i| (i.to_string().into_bytes(), *i))
+                    .collect()
             } else {
-                headers[i].to_vec()
+                sel.iter()
+                    .zip(sel.select(headers))
+                    .map(|(i, h)| (h.to_vec(), *i))
+                    .collect()
             };
 
-            Ok((col_name, i))
+            Ok(info)
         })
-        .collect::<Result<Vec<(Vec<u8>, usize)>, CliError>>()?;
+        .transpose()?
+        .unwrap_or_else(Vec::new);
+
+    let has_added_series = !additional_series_indices.is_empty();
+
+    if args.flag_category.is_some() && has_added_series {
+        Err("-c, --category cannot work when multiple columns are given to <y>!")?;
+    }
+
+    if args.flag_regression_line {
+        if args.flag_bars || args.flag_line {
+            Err("-R/--regression-line does not work with -B/--bars nor -L/--line!")?;
+        }
+
+        if args.flag_category.is_some() || !has_added_series {
+            Err("-R/--regression-line only works with single series (e.g. when using -c/--category or when selecting multiple columns with <y>)!")?;
+        }
+    }
+
+    let share_x_scale = args.flag_share_x_scale == "yes";
+    let share_y_scale = args
+        .flag_share_y_scale
+        .as_ref()
+        .map(|choice| choice == "yes")
+        .unwrap_or(!has_added_series);
 
     let showing_multiple_series =
         category_column_index.is_some() || !additional_series_indices.is_empty();
 
-    let mut record = csv::ByteRecord::new();
+    let mut record = simd_csv::ByteRecord::new();
 
     let mut series_builder = if !additional_series_indices.is_empty() {
         let mut multiple_series = MultipleSeries::with_capacity(additional_series_indices.len());
@@ -439,7 +482,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             Some(y_column_index) => {
                 try_parse_as_float!(args.flag_y_scale, &record[y_column_index])
             }
-            None => args.flag_y_scale.convert(1.0),
+            None => 1.0,
         };
 
         // Filtering out-of-bounds values
@@ -481,7 +524,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     for (_, series) in finalized_series.iter_mut() {
         if args.flag_time {
-            series.mark_as_temporal(args.flag_granularity.map(|g| g.into_inner()));
+            series.mark_as_temporal(
+                args.flag_granularity.map(|g| g.into_inner()),
+                args.flag_aggregate.as_deref().unwrap_or("sum(_)"),
+            )?;
         }
 
         // Domain bounds
@@ -694,6 +740,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         }
 
                         // Create the datasets to fill the chart with
+                        let can_display_y_axis_title =
+                            single_finalized_series.1.can_display_y_axis_title();
+
                         let single_finalized_series = (
                             single_finalized_series.0.clone(),
                             single_finalized_series
@@ -726,7 +775,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                             cols / grid_cols,
                         );
                         let y_axis = Axis::default()
-                            .title(if y_axis_info.can_be_displayed {
+                            .title(if can_display_y_axis_title {
                                 if has_added_series {
                                     single_finalized_series.0.unwrap().dim()
                                 } else {
@@ -876,11 +925,11 @@ fn fix_flat_domain(domain: (f64, f64), axis_type: AxisType) -> (f64, f64) {
             let center_value = domain.0;
             (center_value - 1.0, center_value + 1.0)
         }
-        _ => unimplemented!(),
+        _ => domain, // We can do better but la flemme...
     }
 }
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 struct AxisInfo {
     can_be_displayed: bool,
     scale: Scale,
@@ -903,8 +952,8 @@ impl AxisInfo {
         let mut y_domain = first_series.y_domain().unwrap();
         let mut x_axis_type = first_series.types.0;
         let mut y_axis_type = first_series.types.1;
-        let mut x_can_be_displayed = true;
-        let mut y_can_be_displayed = true;
+        let mut x_can_be_displayed = first_series.can_display_x_axis_title();
+        let mut y_can_be_displayed = first_series.can_display_y_axis_title();
 
         for (_, other_series) in series {
             let other_x_domain = other_series.x_domain().unwrap();
@@ -1097,6 +1146,7 @@ impl AxisType {
     }
 }
 
+#[derive(Debug)]
 struct Series {
     types: (AxisType, AxisType),
     points: Vec<(f64, f64)>,
@@ -1194,7 +1244,7 @@ impl Series {
         }
     }
 
-    fn mark_as_temporal(&mut self, granularity: Option<Unit>) {
+    fn mark_as_temporal(&mut self, granularity: Option<Unit>, expr: &str) -> CliResult<()> {
         if let Some((x_domain, y_domain)) = self.extent.as_mut() {
             let granularity = granularity.unwrap_or_else(|| {
                 infer_temporal_granularity(
@@ -1205,20 +1255,27 @@ impl Series {
             });
             self.types.0 = AxisType::Timestamp(granularity);
 
-            let mut buckets: HashMap<i64, f64> = HashMap::new();
+            let mut buckets = GroupAggregationProgram::<i64>::parse_without_headers(expr)?;
 
-            for (x, y) in self.points.iter() {
-                buckets
-                    .entry(floor_timestamp(*x, granularity))
-                    .and_modify(|c| *c += *y)
-                    .or_insert(*y);
+            if !buckets.has_single_expr() {
+                Err(format!(
+                    "-A, --aggregate should only have a single clause but found {} instead!",
+                    buckets.len()
+                ))?;
+            }
+
+            for (index, (x, y)) in self.points.iter().enumerate() {
+                buckets.run_with(floor_timestamp(*x, granularity), index, *y)?;
             }
 
             self.points.clear();
 
             let mut new_y_domain: Option<(f64, f64)> = None;
 
-            for (x, y) in buckets {
+            for result in buckets.iter() {
+                let (x, y) = result?;
+                let y = y.try_as_f64()?;
+
                 match new_y_domain.as_mut() {
                     None => new_y_domain = Some((y, y)),
                     Some((current_y_min, current_y_max)) => {
@@ -1241,6 +1298,8 @@ impl Series {
             );
             *y_domain = new_y_domain.unwrap();
         }
+
+        Ok(())
     }
 
     fn set_x_min(&mut self, v: f64) {
