@@ -8,10 +8,18 @@ use numfmt::{Formatter, Precision};
 use unicode_width::UnicodeWidthStr;
 
 use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
-use crate::util::{self, ImmutableRecordHelpers};
+use crate::select::SelectedColumns;
+use crate::util::{self, ColorMode};
 use crate::CliResult;
 
+fn prepend(record: &csv::StringRecord, cell_value: &str) -> csv::StringRecord {
+    let mut new_record = csv::StringRecord::new();
+    new_record.push_field(cell_value);
+    new_record.extend(record);
+
+    new_record
+}
+
 const HEADERS_ROWS: usize = 8;
 
 type BoxCharsArray = [char; 11];
@@ -191,9 +199,9 @@ the -p/--pager flag that internally rely on the ubiquitous \"less\"
 command.
 
 If you still want to use a pager manually, don't forget to use
-the -e/--expand and -C/--force-colors flags before piping like so:
+the -e/--expand and --color=always flags before piping like so:
 
-    $ xan view -eC file.csv | less -SR
+    $ xan view -e --color=always file.csv | less -SR
 
 Finally, it is possible to customize the default behavior of this command through
 the \"XAN_VIEW_ARGS\" environment variable. This variable takes a series of
@@ -227,8 +235,11 @@ view options:
                             Defaults to using all your terminal's width or 80 if
                             terminal's size cannot be found (i.e. when piping to file).
                             Can also be given as a ratio of the terminal's width e.g. \"0.5\".
-    -C, --force-colors      Force colors even if output is not supposed to be able to
-                            handle them.
+    --color <when>          When to color the output using ANSI escape codes.
+                            Use `auto` for automatic detection, `never` to
+                            disable colors completely and `always` to force
+                            colors, even when the output could not handle them.
+                            [default: auto]
     -e, --expand            Expand the table so that in can be easily piped to
                             a pager such as \"less\", with larger width constraints.
     -E, --sanitize-emojis   Replace emojis by their shortcode to avoid formatting issues.
@@ -251,13 +262,13 @@ Common options:
 #[derive(Deserialize, Debug)]
 struct Args {
     arg_input: Option<String>,
-    flag_select: SelectColumns,
+    flag_select: SelectedColumns,
     flag_pager: bool,
     flag_theme: String,
     flag_cols: Option<String>,
     flag_delimiter: Option<Delimiter>,
     flag_no_headers: bool,
-    flag_force_colors: bool,
+    flag_color: ColorMode,
     flag_all: bool,
     flag_limit: usize,
     flag_rainbow: bool,
@@ -266,8 +277,8 @@ struct Args {
     flag_hide_index: bool,
     flag_hide_headers: bool,
     flag_hide_info: bool,
-    flag_groupby: Option<SelectColumns>,
-    flag_right: Option<SelectColumns>,
+    flag_groupby: Option<SelectedColumns>,
+    flag_right: Option<SelectedColumns>,
     flag_significance: Option<NonZeroUsize>,
 }
 
@@ -280,16 +291,16 @@ impl Args {
         if self.flag_no_headers {
             self.flag_hide_headers = true;
         }
+
+        if self.flag_pager {
+            self.flag_color = ColorMode::Always;
+        }
     }
 
     fn infer_expand(&self) -> bool {
         self.flag_pager || self.flag_expand
     }
 
-    fn infer_force_colors(&self) -> bool {
-        self.flag_pager || self.flag_force_colors
-    }
-
     fn merge(from_env: Self, mut from_argv: Self) -> Self {
         if from_argv.flag_theme == "table" && from_env.flag_theme != "table" {
             from_argv.flag_theme = from_env.flag_theme;
@@ -330,6 +341,7 @@ impl Args {
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut args: Args = util::get_args(USAGE, argv)?;
     args.resolve();
+    args.flag_color.apply();
 
     let mut env_var_argv = vec!["xan", "view"];
     let env_var_split =
@@ -342,11 +354,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut env_args: Args = util::get_args(USAGE, &env_var_argv)?;
     env_args.resolve();
 
-    let args = Args::merge(env_args, args);
-
-    if args.infer_force_colors() {
-        colored::control::set_override(true);
-    }
+    let mut args = Args::merge(env_args, args);
 
     let emoji_sanitizer = util::EmojiSanitizer::new();
 
@@ -366,6 +374,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .no_headers(args.flag_no_headers)
         .select(args.flag_select.clone());
 
+    if rconfig.no_headers {
+        args.flag_hide_headers = true;
+    }
+
     let mut rdr = rconfig.reader()?;
     let byte_headers = rdr.byte_headers()?.clone();
     let mut sel = rconfig.selection(&byte_headers)?;
@@ -375,7 +387,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut groupby_sel_opt = args
         .flag_groupby
         .clone()
-        .map(|cols| cols.selection(&byte_headers, !args.flag_no_headers))
+        .map(|cols| cols.selection(&byte_headers, !rconfig.no_headers))
         .transpose()?;
 
     if let Some(groupby_sel) = &groupby_sel_opt {
@@ -389,7 +401,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     groupby_sel_opt = args
         .flag_groupby
         .clone()
-        .map(|cols| cols.selection(&sel.select(&byte_headers).collect(), !args.flag_no_headers))
+        .map(|cols| {
+            cols.selection(
+                &sel.select(&byte_headers).collect::<csv::ByteRecord>(),
+                !rconfig.no_headers,
+            )
+        })
         .transpose()?;
 
     if let (Some(groupby_sel), false) = (&mut groupby_sel_opt, args.flag_hide_index) {
@@ -402,11 +419,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut right_sel_opt = args
         .flag_right
         .as_ref()
-        .map(|s| s.selection(headers.as_byte_record(), !args.flag_no_headers))
+        .map(|s| s.selection(headers.as_byte_record(), !rconfig.no_headers))
         .transpose()?;
 
     if !args.flag_hide_index {
-        headers = headers.prepend(theme.index_column_header);
+        headers = prepend(&headers, theme.index_column_header);
 
         if let Some(right_sel) = &mut right_sel_opt {
             right_sel.offset_by(1);
@@ -429,6 +446,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             .collect();
     }
 
+    headers = headers
+        .into_iter()
+        .map(util::sanitize_text_for_single_line_printing)
+        .collect();
+
     let mut all_records_buffered = false;
 
     let mut number_formatter = args.flag_significance.map(|s| {
@@ -471,7 +493,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         .collect::<csv::StringRecord>();
 
                     if !args.flag_hide_index {
-                        record = record.prepend(&i.to_string());
+                        record = prepend(&record, &i.to_string());
                     }
 
                     records.push(record);
@@ -517,7 +539,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         })
         .collect();
 
-    // Alignment inferrence
+    // Alignment inference
     // TODO: type tagging for values should happen on initial read
     let right_sel_mask_opt = right_sel_opt.map(|sel| sel.mask(headers.len()));
 
@@ -541,7 +563,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         })
         .collect::<Vec<_>>();
 
-    // Width inferrence
+    // Width inference
     let displayed_columns = infer_best_column_display(
         cols,
         &max_column_widths,
@@ -731,7 +753,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             .map(|col| (col, &headers[col.index]))
             .enumerate()
             .map(|(i, (col, h))| {
-                let cell = util::unicode_aware_rpad_with_ellipsis(h, col.allowed_width, " ");
+                let cell = util::unicode_aware_highlighted_pad_with_ellipsis(
+                    false,
+                    h,
+                    col.allowed_width,
+                    " ",
+                    true,
+                );
 
                 if !args.flag_hide_index && i == 0 {
                     cell.dimmed()
