@@ -11,11 +11,12 @@ use std::num::NonZeroUsize;
 use std::rc::Rc;
 
 use bstr::ByteSlice;
+use simd_csv::ByteRecord;
 
 use crate::collections::ClusteredInsertHashmap;
 use crate::collections::{hash_map::Entry, HashMap};
 use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
+use crate::select::SelectedColumns;
 use crate::util;
 use crate::CliError;
 use crate::CliResult;
@@ -81,7 +82,7 @@ This command can compute 5 kinds of differents vocabulary statistics:
 
 4. doc-token-level statistics (using the \"doc-token\" subcommand):
     - (*doc): columns representing the document (named like the input)
-    - token: some distinct documnet token (the column will be named like the input)
+    - token: some distinct document token (the column will be named like the input)
     - tf: term frequency for the token in the document
     - expected_tf: expected absolute term frequency (does not follow --tf-weight)
     - tfidf: term frequency * idf for the token in the document
@@ -185,8 +186,8 @@ struct Args {
     cmd_corpus: bool,
     cmd_cooc: bool,
     arg_input: Option<String>,
-    flag_token: Option<SelectColumns>,
-    flag_doc: Option<SelectColumns>,
+    flag_token: Option<SelectedColumns>,
+    flag_doc: Option<SelectedColumns>,
     flag_sep: Option<String>,
     flag_implode: bool,
     flag_tf_weight: TfWeighting,
@@ -238,24 +239,24 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers);
 
-    let mut rdr = rconf.reader()?;
+    let mut rdr = rconf.simd_reader()?;
     let headers = rdr.byte_headers()?.clone();
 
     let flag_token = args.flag_token.unwrap_or_else(|| {
-        SelectColumns::parse(if args.flag_implode { "token" } else { "tokens" }).unwrap()
+        SelectedColumns::parse(if args.flag_implode { "token" } else { "tokens" }).unwrap()
     });
 
-    let token_pos = flag_token.single_selection(&headers, !args.flag_no_headers)?;
+    let token_pos = flag_token.single_selection(&headers, !rconf.no_headers)?;
 
     let doc_sel = args
         .flag_doc
-        .map(|s| s.selection(&headers, !args.flag_no_headers))
+        .map(|s| s.selection(&headers, !rconf.no_headers))
         .transpose()?;
 
-    let mut record = csv::ByteRecord::new();
+    let mut record = ByteRecord::new();
     let mut i: usize = 0;
 
-    let mut wtr = Config::new(&args.flag_output).writer()?;
+    let mut wtr = Config::new(&args.flag_output).simd_writer()?;
 
     if args.cmd_cooc {
         let mut cooccurrences = Cooccurrences::default();
@@ -282,7 +283,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     ClusteredInsertHashmap::new();
 
                 while rdr.read_byte_record(&mut record)? {
-                    let doc = sel.collect(&record);
+                    let doc = sel.select(&record).collect();
 
                     doc_tokens.insert_with_or_else(
                         doc,
@@ -376,7 +377,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 let mut current_opt: Option<(Document, Vec<Rc<Token>>)> = None;
 
                 while rdr.read_byte_record(&mut record)? {
-                    let doc = sel.collect(&record);
+                    let doc = sel.select(&record).collect();
                     let token = Rc::new(record[token_pos].to_vec());
 
                     match current_opt.as_mut() {
@@ -450,8 +451,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     while rdr.read_byte_record(&mut record)? {
         let document: Document = match &doc_sel {
-            Some(sel) => sel.select(&record).map(|cell| cell.to_vec()).collect(),
-            None => vec![i.to_string().into_bytes()],
+            Some(sel) => sel.select(&record).collect(),
+            None => {
+                let mut doc_key = ByteRecord::new();
+                doc_key.push_field(i.to_string().as_bytes());
+                doc_key
+            }
         };
 
         if let Some(sep) = &args.flag_sep {
@@ -486,7 +491,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         wtr.write_record(headers)?;
         vocab.for_each_token_level_record(|r| wtr.write_byte_record(r))?;
     } else if args.cmd_doc_token {
-        let mut output_headers = csv::ByteRecord::new();
+        let mut output_headers = ByteRecord::new();
 
         if let Some(sel) = &doc_sel {
             for col_name in sel.select(&headers) {
@@ -512,7 +517,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             |r| wtr.write_byte_record(r),
         )?;
     } else if args.cmd_doc {
-        let mut output_headers = csv::ByteRecord::new();
+        let mut output_headers = ByteRecord::new();
 
         if let Some(sel) = &doc_sel {
             for col_name in sel.select(&headers) {
@@ -548,7 +553,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     Ok(wtr.flush()?)
 }
 
-type Document = Vec<Vec<u8>>;
+type Document = ByteRecord;
 type Token = Vec<u8>;
 type TokenID = usize;
 
@@ -755,15 +760,15 @@ impl Vocabulary {
 
     fn for_each_doc_level_record<F, E>(self, mut callback: F) -> Result<(), E>
     where
-        F: FnMut(&csv::ByteRecord) -> Result<(), E>,
+        F: FnMut(&ByteRecord) -> Result<(), E>,
     {
-        let mut record = csv::ByteRecord::new();
+        let mut record = ByteRecord::new();
 
         for (doc, doc_stats) in self.documents.into_iter() {
             record.clear();
 
-            for cell in doc {
-                record.push_field(&cell);
+            for cell in doc.into_iter() {
+                record.push_field(cell);
             }
 
             record.push_field(doc_stats.doc_len().to_string().as_bytes());
@@ -777,7 +782,7 @@ impl Vocabulary {
 
     fn for_each_token_level_record<F, E>(self, mut callback: F) -> Result<(), E>
     where
-        F: FnMut(&csv::ByteRecord) -> Result<(), E>,
+        F: FnMut(&ByteRecord) -> Result<(), E>,
     {
         let n = self.doc_count();
 
@@ -785,7 +790,7 @@ impl Vocabulary {
             return Ok(());
         }
 
-        let mut record = csv::ByteRecord::new();
+        let mut record = ByteRecord::new();
 
         for stats in self.tokens.into_iter() {
             record.clear();
@@ -812,7 +817,7 @@ impl Vocabulary {
         mut callback: F,
     ) -> Result<(), E>
     where
-        F: FnMut(&csv::ByteRecord) -> Result<(), E>,
+        F: FnMut(&ByteRecord) -> Result<(), E>,
     {
         let n = self.doc_count();
 
@@ -823,7 +828,7 @@ impl Vocabulary {
         let average_doc_len = self.average_doc_len();
 
         // Aggregating stats for bm25 and chi2
-        let mut record = csv::ByteRecord::new();
+        let mut record = ByteRecord::new();
 
         for (doc, doc_stats) in self.documents.into_iter() {
             let doc_len = doc_stats.doc_len();
@@ -1215,9 +1220,9 @@ impl Cooccurrences {
         mut callback: F,
     ) -> Result<(), E>
     where
-        F: FnMut(&csv::ByteRecord) -> Result<(), E>,
+        F: FnMut(&ByteRecord) -> Result<(), E>,
     {
-        let mut csv_record = csv::ByteRecord::new();
+        let mut csv_record = ByteRecord::new();
         let n = self.cooccurrences_count;
 
         for source_entry in self.token_entries.iter() {
@@ -1281,7 +1286,7 @@ impl Cooccurrences {
     // NOTE: currently we avoid self loops because they are fiddly
     fn for_each_distrib_cooc_record<F, E>(self, min_count: usize, mut callback: F) -> Result<(), E>
     where
-        F: FnMut(&csv::ByteRecord) -> Result<(), E>,
+        F: FnMut(&ByteRecord) -> Result<(), E>,
     {
         #[derive(Default)]
         struct Metrics {
@@ -1289,7 +1294,7 @@ impl Cooccurrences {
             g2: f64,
         }
 
-        let mut csv_record = csv::ByteRecord::new();
+        let mut csv_record = ByteRecord::new();
         let n = self.cooccurrences_count;
 
         let mut sums: Vec<Metrics> = Vec::with_capacity(self.token_entries.len());
@@ -1428,9 +1433,9 @@ impl Cooccurrences {
         mut callback: F,
     ) -> Result<(), E>
     where
-        F: FnMut(&csv::ByteRecord) -> Result<(), E>,
+        F: FnMut(&ByteRecord) -> Result<(), E>,
     {
-        let mut csv_record = csv::ByteRecord::new();
+        let mut csv_record = ByteRecord::new();
         let n = self.cooccurrences_count;
 
         let g2_significance = g2_significance.unwrap_or(3.84);
