@@ -1,6 +1,7 @@
-use std::collections::HashMap;
 use std::hash::Hash;
 
+use crate::collections::HashMap;
+
 #[derive(Debug)]
 pub struct IncrementalId<K> {
     map: HashMap<K, usize>,
